@@ -192,6 +192,9 @@ typedef struct _GstAudioDecoderContext
   gboolean eos;
   gboolean sync;
 
+  gboolean had_output_data;
+  gboolean had_input_data;
+
   /* misc */
   gint delay;
 
@@ -276,6 +279,69 @@ struct _GstAudioDecoderPrivate
   GList *pending_events;
 };
 
+//* Default channel layouts taken from audioconvert */
+static const GstAudioChannelPosition default_positions[8][8] = {
+  /* 1 channel */
+  {
+        GST_AUDIO_CHANNEL_POSITION_MONO,
+      },
+  /* 2 channels */
+  {
+        GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
+      },
+  /* 3 channels (2.1) */
+  {
+        GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
+        GST_AUDIO_CHANNEL_POSITION_LFE1,
+      },
+  /* 4 channels (4.0) */
+  {
+        GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
+        GST_AUDIO_CHANNEL_POSITION_REAR_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT,
+      },
+  /* 5 channels */
+  {
+        GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
+        GST_AUDIO_CHANNEL_POSITION_REAR_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
+      },
+  /* 6 channels (5.1) */
+  {
+        GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
+        GST_AUDIO_CHANNEL_POSITION_REAR_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
+        GST_AUDIO_CHANNEL_POSITION_LFE1,
+      },
+  /* 7 channels (6.1) */
+  {
+        GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
+        GST_AUDIO_CHANNEL_POSITION_REAR_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
+        GST_AUDIO_CHANNEL_POSITION_LFE1,
+        GST_AUDIO_CHANNEL_POSITION_REAR_CENTER,
+      },
+  /* 8 channels (7.1) */
+  {
+        GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
+        GST_AUDIO_CHANNEL_POSITION_REAR_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
+        GST_AUDIO_CHANNEL_POSITION_LFE1,
+        GST_AUDIO_CHANNEL_POSITION_SIDE_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_SIDE_RIGHT,
+      }
+};
 
 static void gst_audio_decoder_finalize (GObject * object);
 static void gst_audio_decoder_set_property (GObject * object,
@@ -312,6 +378,7 @@ static gboolean gst_audio_decoder_decide_allocation_default (GstAudioDecoder *
 static gboolean gst_audio_decoder_propose_allocation_default (GstAudioDecoder *
     dec, GstQuery * query);
 static gboolean gst_audio_decoder_negotiate_default (GstAudioDecoder * dec);
+static gboolean gst_audio_decoder_negotiate_unlocked (GstAudioDecoder * dec);
 
 static GstElementClass *parent_class = NULL;
 
@@ -494,6 +561,8 @@ gst_audio_decoder_reset (GstAudioDecoder * dec, gboolean full)
 
     gst_audio_info_init (&dec->priv->ctx.info);
     dec->priv->ctx.max_errors = GST_AUDIO_DECODER_MAX_ERRORS;
+    dec->priv->ctx.had_output_data = FALSE;
+    dec->priv->ctx.had_input_data = FALSE;
   }
 
   g_queue_foreach (&dec->priv->frames, (GFunc) gst_buffer_unref, NULL);
@@ -651,11 +720,25 @@ no_decide_allocation:
   }
 }
 
+static gboolean
+gst_audio_decoder_negotiate_unlocked (GstAudioDecoder * dec)
+{
+  GstAudioDecoderClass *klass = GST_AUDIO_DECODER_GET_CLASS (dec);
+  gboolean ret = TRUE;
+
+  if (G_LIKELY (klass->negotiate))
+    ret = klass->negotiate (dec);
+
+  return ret;
+}
+
 /**
  * gst_audio_decoder_negotiate:
  * @dec: a #GstAudioDecoder
  *
- * Negotiate with downstreame elements to currently configured #GstAudioInfo.
+ * Negotiate with downstream elements to currently configured #GstAudioInfo.
+ * Unmark GST_PAD_FLAG_NEED_RECONFIGURE in any case. But mark it again if
+ * negotiate fails.
  *
  * Returns: #TRUE if the negotiation succeeded, else #FALSE.
  */
@@ -670,8 +753,12 @@ gst_audio_decoder_negotiate (GstAudioDecoder * dec)
   klass = GST_AUDIO_DECODER_GET_CLASS (dec);
 
   GST_AUDIO_DECODER_STREAM_LOCK (dec);
-  if (klass->negotiate)
+  gst_pad_check_reconfigure (dec->srcpad);
+  if (klass->negotiate) {
     res = klass->negotiate (dec);
+    if (!res)
+      gst_pad_mark_reconfigure (dec->srcpad);
+  }
   GST_AUDIO_DECODER_STREAM_UNLOCK (dec);
 
   return res;
@@ -814,6 +901,7 @@ gst_audio_decoder_push_forward (GstAudioDecoder * dec, GstBuffer * buf)
   GstAudioDecoderPrivate *priv;
   GstAudioDecoderContext *ctx;
   GstFlowReturn ret = GST_FLOW_OK;
+  GstClockTime ts;
 
   klass = GST_AUDIO_DECODER_GET_CLASS (dec);
   priv = dec->priv;
@@ -826,6 +914,9 @@ gst_audio_decoder_push_forward (GstAudioDecoder * dec, GstBuffer * buf)
     return GST_FLOW_OK;
   }
 
+  ctx->had_output_data = TRUE;
+  ts = GST_BUFFER_TIMESTAMP (buf);
+
   GST_LOG_OBJECT (dec,
       "clipping buffer of size %" G_GSIZE_FORMAT " with ts %" GST_TIME_FORMAT
       ", duration %" GST_TIME_FORMAT, gst_buffer_get_size (buf),
@@ -837,6 +928,12 @@ gst_audio_decoder_push_forward (GstAudioDecoder * dec, GstBuffer * buf)
       ctx->info.bpf);
   if (G_UNLIKELY (!buf)) {
     GST_DEBUG_OBJECT (dec, "no data after clipping to segment");
+    if (dec->output_segment.rate >= 0) {
+      if (ts >= dec->output_segment.stop)
+        ret = GST_FLOW_EOS;
+    } else if (ts < dec->output_segment.start) {
+      ret = GST_FLOW_EOS;
+    }
     goto exit;
   }
 
@@ -981,6 +1078,31 @@ send_pending_events (GstAudioDecoder * dec)
   g_list_free (pending_events);
 }
 
+static GstFlowReturn
+check_pending_reconfigure (GstAudioDecoder * dec)
+{
+  GstFlowReturn ret = GST_FLOW_OK;
+  GstAudioDecoderContext *ctx;
+  gboolean needs_reconfigure;
+
+  ctx = &dec->priv->ctx;
+
+  needs_reconfigure = gst_pad_check_reconfigure (dec->srcpad);
+  if (G_UNLIKELY (ctx->output_format_changed ||
+          (GST_AUDIO_INFO_IS_VALID (&ctx->info)
+              && needs_reconfigure))) {
+    if (!gst_audio_decoder_negotiate_unlocked (dec)) {
+      gst_pad_mark_reconfigure (dec->srcpad);
+      if (GST_PAD_IS_FLUSHING (dec->srcpad))
+        ret = GST_FLOW_FLUSHING;
+      else
+        ret = GST_FLOW_NOT_NEGOTIATED;
+    }
+  }
+  return ret;
+}
+
+
 /**
  * gst_audio_decoder_finish_frame:
  * @dec: a #GstAudioDecoder
@@ -1032,21 +1154,13 @@ gst_audio_decoder_finish_frame (GstAudioDecoder * dec, GstBuffer * buf,
 
   GST_AUDIO_DECODER_STREAM_LOCK (dec);
 
-  if (buf && G_UNLIKELY (ctx->output_format_changed ||
-          (GST_AUDIO_INFO_IS_VALID (&ctx->info)
-              && gst_pad_check_reconfigure (dec->srcpad)))) {
-    if (!gst_audio_decoder_negotiate (dec)) {
-      gst_pad_mark_reconfigure (dec->srcpad);
-      if (GST_PAD_IS_FLUSHING (dec->srcpad))
-        ret = GST_FLOW_FLUSHING;
-      else
-        ret = GST_FLOW_NOT_NEGOTIATED;
+  if (buf) {
+    ret = check_pending_reconfigure (dec);
+    if (ret == GST_FLOW_FLUSHING || ret == GST_FLOW_NOT_NEGOTIATED)
       goto exit;
-    }
-  }
 
-  if (buf && priv->pending_events) {
-    send_pending_events (dec);
+    if (priv->pending_events)
+      send_pending_events (dec);
   }
 
   /* output shoud be whole number of sample frames */
@@ -1059,16 +1173,21 @@ gst_audio_decoder_finish_frame (GstAudioDecoder * dec, GstBuffer * buf,
 
   /* frame and ts book-keeping */
   if (G_UNLIKELY (frames < 0)) {
-    if (G_UNLIKELY (-frames - 1 > priv->frames.length))
-      goto overflow;
-    frames = priv->frames.length + frames + 1;
+    if (G_UNLIKELY (-frames - 1 > priv->frames.length)) {
+      GST_ELEMENT_WARNING (dec, STREAM, ENCODE,
+          ("received more decoded frames %d than provided %d", frames,
+              priv->frames.length), (NULL));
+      frames = 0;
+    } else {
+      frames = priv->frames.length + frames + 1;
+    }
   } else if (G_UNLIKELY (frames > priv->frames.length)) {
     if (G_LIKELY (!priv->force)) {
-      /* no way we can let this pass */
-      g_assert_not_reached ();
-      /* really no way */
-      goto overflow;
+      GST_ELEMENT_WARNING (dec, STREAM, ENCODE,
+          ("received more decoded frames %d than provided %d", frames,
+              priv->frames.length), (NULL));
     }
+    frames = priv->frames.length;
   }
 
   if (G_LIKELY (priv->frames.length))
@@ -1177,16 +1296,6 @@ wrong_buffer:
     ret = GST_FLOW_ERROR;
     goto exit;
   }
-overflow:
-  {
-    GST_ELEMENT_ERROR (dec, STREAM, ENCODE,
-        ("received more decoded frames %d than provided %d", frames,
-            priv->frames.length), (NULL));
-    if (buf)
-      gst_buffer_unref (buf);
-    ret = GST_FLOW_ERROR;
-    goto exit;
-  }
 }
 
 static GstFlowReturn
@@ -1621,6 +1730,21 @@ gst_audio_decoder_chain_reverse (GstAudioDecoder * dec, GstBuffer * buf)
   return result;
 }
 
+static gboolean
+gst_audio_decoder_do_caps (GstAudioDecoder * dec)
+{
+  GstCaps *caps = gst_pad_get_current_caps (dec->sinkpad);
+  if (caps) {
+    if (!gst_audio_decoder_sink_setcaps (dec, caps)) {
+      gst_caps_unref (caps);
+      return FALSE;
+    }
+    gst_caps_unref (caps);
+  }
+  dec->priv->do_caps = FALSE;
+  return TRUE;
+}
+
 static GstFlowReturn
 gst_audio_decoder_chain (GstPad * pad, GstObject * parent, GstBuffer * buffer)
 {
@@ -1630,15 +1754,9 @@ gst_audio_decoder_chain (GstPad * pad, GstObject * parent, GstBuffer * buffer)
   dec = GST_AUDIO_DECODER (parent);
 
   if (G_UNLIKELY (dec->priv->do_caps)) {
-    GstCaps *caps = gst_pad_get_current_caps (dec->sinkpad);
-    if (caps) {
-      if (!gst_audio_decoder_sink_setcaps (dec, caps)) {
-        gst_caps_unref (caps);
-        goto not_negotiated;
-      }
-      gst_caps_unref (caps);
+    if (!gst_audio_decoder_do_caps (dec)) {
+      goto not_negotiated;
     }
-    dec->priv->do_caps = FALSE;
   }
 
   if (G_UNLIKELY (!gst_pad_has_current_caps (pad) && dec->priv->needs_format))
@@ -1652,6 +1770,8 @@ gst_audio_decoder_chain (GstPad * pad, GstObject * parent, GstBuffer * buffer)
 
   GST_AUDIO_DECODER_STREAM_LOCK (dec);
 
+  dec->priv->ctx.had_input_data = TRUE;
+
   if (GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_DISCONT)) {
     gint64 samples, ts;
 
@@ -1702,6 +1822,197 @@ gst_audio_decoder_do_byte (GstAudioDecoder * dec)
       dec->priv->ctx.info.rate <= dec->priv->samples_out;
 }
 
+/* Must be called holding the GST_AUDIO_DECODER_STREAM_LOCK */
+static gboolean
+gst_audio_decoder_negotiate_default_caps (GstAudioDecoder * dec)
+{
+  GstCaps *caps;
+  gint i;
+  gint channels = 0;
+  gint rate;
+  guint64 channel_mask = 0;
+  gint caps_size;
+  GstStructure *structure;
+
+  caps = gst_pad_get_current_caps (dec->srcpad);
+  if (caps && !gst_audio_info_from_caps (&dec->priv->ctx.info, caps))
+    goto caps_error;
+  if (caps)
+    gst_caps_unref (caps);
+
+  caps = gst_pad_get_allowed_caps (dec->srcpad);
+  if (!caps || gst_caps_is_empty (caps) || gst_caps_is_any (caps))
+    goto caps_error;
+
+  /* before fixating, try to use whatever upstream provided */
+  caps = gst_caps_make_writable (caps);
+  caps_size = gst_caps_get_size (caps);
+  if (dec->priv->ctx.input_caps) {
+    GstCaps *sinkcaps = dec->priv->ctx.input_caps;
+    GstStructure *structure = gst_caps_get_structure (sinkcaps, 0);
+
+    if (gst_structure_get_int (structure, "rate", &rate)) {
+      for (i = 0; i < caps_size; i++) {
+        gst_structure_set (gst_caps_get_structure (caps, i), "rate",
+            G_TYPE_INT, rate, NULL);
+      }
+    }
+
+    if (gst_structure_get_int (structure, "channels", &channels)) {
+      for (i = 0; i < caps_size; i++) {
+        gst_structure_set (gst_caps_get_structure (caps, i), "channels",
+            G_TYPE_INT, channels, NULL);
+      }
+    }
+
+    if (gst_structure_get (structure, "channel-mask", GST_TYPE_BITMASK,
+            &channel_mask, NULL)) {
+      for (i = 0; i < caps_size; i++) {
+        gst_structure_set (gst_caps_get_structure (caps, i), "channel-mask",
+            GST_TYPE_BITMASK, channel_mask, NULL);
+      }
+    }
+  }
+
+  for (i = 0; i < caps_size; i++) {
+    structure = gst_caps_get_structure (caps, i);
+    gst_structure_fixate_field_nearest_int (structure,
+        "channels", GST_AUDIO_DEF_CHANNELS);
+    gst_structure_fixate_field_nearest_int (structure,
+        "rate", GST_AUDIO_DEF_RATE);
+  }
+  caps = gst_caps_fixate (caps);
+  structure = gst_caps_get_structure (caps, 0);
+
+  /* Need to add a channel-mask if channels > 2 */
+  gst_structure_get_int (structure, "channels", &channels);
+  if (channels > 2 && !gst_structure_has_field (structure, "channel-mask")) {
+    if (channels <= 8) {
+      channel_mask = 0;
+      for (i = 0; i < channels; i++)
+        channel_mask |=
+            G_GUINT64_CONSTANT (1) << default_positions[channels - 1][i];
+
+      gst_structure_set (structure, "channel-mask",
+          GST_TYPE_BITMASK, channel_mask, NULL);
+    } else {
+      GST_WARNING_OBJECT (dec, "No default channel-mask for %d channels",
+          channels);
+    }
+  }
+
+  if (!caps || !gst_audio_info_from_caps (&dec->priv->ctx.info, caps))
+    goto caps_error;
+
+  GST_INFO_OBJECT (dec,
+      "Chose default caps %" GST_PTR_FORMAT " for initial gap", caps);
+  gst_caps_unref (caps);
+
+  if (!gst_audio_decoder_negotiate_unlocked (dec)) {
+    GST_INFO_OBJECT (dec, "Failed to negotiate default caps for initial gap");
+    gst_pad_mark_reconfigure (dec->srcpad);
+    return FALSE;
+  }
+
+  return TRUE;
+
+caps_error:
+  {
+    if (caps)
+      gst_caps_unref (caps);
+    return FALSE;
+  }
+}
+
+static gboolean
+gst_audio_decoder_handle_gap (GstAudioDecoder * dec, GstEvent * event)
+{
+  gboolean ret;
+  GstClockTime timestamp, duration;
+
+  /* Check if there is a caps pending to be pushed */
+  if (G_UNLIKELY (dec->priv->do_caps)) {
+    if (!gst_audio_decoder_do_caps (dec)) {
+      goto not_negotiated;
+    }
+  }
+
+  /* Ensure we have caps first */
+  GST_AUDIO_DECODER_STREAM_LOCK (dec);
+  if (!GST_AUDIO_INFO_IS_VALID (&dec->priv->ctx.info)) {
+    if (!gst_audio_decoder_negotiate_default_caps (dec)) {
+      GST_ELEMENT_ERROR (dec, STREAM, FORMAT, (NULL),
+          ("Decoder output not negotiated before GAP event."));
+      GST_AUDIO_DECODER_STREAM_UNLOCK (dec);
+      return FALSE;
+    }
+  }
+  GST_AUDIO_DECODER_STREAM_UNLOCK (dec);
+
+  gst_event_parse_gap (event, &timestamp, &duration);
+
+  /* time progressed without data, see if we can fill the gap with
+   * some concealment data */
+  GST_DEBUG_OBJECT (dec,
+      "gap event: plc %d, do_plc %d, position %" GST_TIME_FORMAT
+      " duration %" GST_TIME_FORMAT,
+      dec->priv->plc, dec->priv->ctx.do_plc,
+      GST_TIME_ARGS (timestamp), GST_TIME_ARGS (duration));
+
+  if (dec->priv->plc && dec->priv->ctx.do_plc && dec->input_segment.rate > 0.0) {
+    GstAudioDecoderClass *klass = GST_AUDIO_DECODER_GET_CLASS (dec);
+    GstBuffer *buf;
+
+    /* hand subclass empty frame with duration that needs covering */
+    buf = gst_buffer_new ();
+    GST_BUFFER_TIMESTAMP (buf) = timestamp;
+    GST_BUFFER_DURATION (buf) = duration;
+    /* best effort, not much error handling */
+    gst_audio_decoder_handle_frame (dec, klass, buf);
+    ret = TRUE;
+    gst_event_unref (event);
+  } else {
+    GstFlowReturn flowret;
+
+    /* sub-class doesn't know how to handle empty buffers,
+     * so just try sending GAP downstream */
+    flowret = check_pending_reconfigure (dec);
+    if (flowret == GST_FLOW_OK) {
+      send_pending_events (dec);
+      ret = gst_audio_decoder_push_event (dec, event);
+    } else {
+      ret = FALSE;
+    }
+  }
+  return ret;
+
+  /* ERRORS */
+not_negotiated:
+  {
+    GST_ELEMENT_ERROR (dec, CORE, NEGOTIATION, (NULL),
+        ("decoder not initialized"));
+    return FALSE;
+  }
+}
+
+static GList *
+_flush_events (GstPad * pad, GList * events)
+{
+  GList *tmp;
+
+  for (tmp = events; tmp; tmp = tmp->next) {
+    if (GST_EVENT_TYPE (tmp->data) == GST_EVENT_EOS ||
+        GST_EVENT_TYPE (tmp->data) == GST_EVENT_SEGMENT ||
+        !GST_EVENT_IS_STICKY (tmp->data)) {
+      gst_event_unref (tmp->data);
+    } else {
+      gst_pad_store_sticky_event (pad, GST_EVENT_CAST (tmp->data));
+    }
+  }
+
+  return NULL;
+}
+
 static gboolean
 gst_audio_decoder_sink_eventfunc (GstAudioDecoder * dec, GstEvent * event)
 {
@@ -1789,47 +2100,16 @@ gst_audio_decoder_sink_eventfunc (GstAudioDecoder * dec, GstEvent * event)
       ret = TRUE;
       break;
     }
-    case GST_EVENT_GAP:{
-      GstClockTime timestamp, duration;
-      gst_event_parse_gap (event, &timestamp, &duration);
-
-      /* time progressed without data, see if we can fill the gap with
-       * some concealment data */
-      GST_DEBUG_OBJECT (dec,
-          "gap event: plc %d, do_plc %d, position %" GST_TIME_FORMAT
-          " duration %" GST_TIME_FORMAT,
-          dec->priv->plc, dec->priv->ctx.do_plc,
-          GST_TIME_ARGS (timestamp), GST_TIME_ARGS (duration));
-      if (dec->priv->plc && dec->priv->ctx.do_plc &&
-          dec->input_segment.rate > 0.0) {
-        GstAudioDecoderClass *klass;
-        GstBuffer *buf;
-
-        klass = GST_AUDIO_DECODER_GET_CLASS (dec);
-        /* hand subclass empty frame with duration that needs covering */
-        buf = gst_buffer_new ();
-        GST_BUFFER_TIMESTAMP (buf) = timestamp;
-        GST_BUFFER_DURATION (buf) = duration;
-        /* best effort, not much error handling */
-        gst_audio_decoder_handle_frame (dec, klass, buf);
-        ret = TRUE;
-        gst_event_unref (event);
-      } else {
-        /* FIXME: sub-class doesn't know how to handle empty buffers,
-         * so just try sending GAP downstream */
-        send_pending_events (dec);
-        ret = gst_audio_decoder_push_event (dec, event);
-      }
+    case GST_EVENT_GAP:
+      ret = gst_audio_decoder_handle_gap (dec, event);
       break;
-    }
     case GST_EVENT_FLUSH_STOP:
       GST_AUDIO_DECODER_STREAM_LOCK (dec);
       /* prepare for fresh start */
       gst_audio_decoder_flush (dec, TRUE);
 
-      g_list_foreach (dec->priv->pending_events, (GFunc) gst_event_unref, NULL);
-      g_list_free (dec->priv->pending_events);
-      dec->priv->pending_events = NULL;
+      dec->priv->pending_events = _flush_events (dec->srcpad,
+          dec->priv->pending_events);
       GST_AUDIO_DECODER_STREAM_UNLOCK (dec);
 
       /* Forward FLUSH_STOP, it is expected to be forwarded immediately
@@ -1842,6 +2122,12 @@ gst_audio_decoder_sink_eventfunc (GstAudioDecoder * dec, GstEvent * event)
       gst_audio_decoder_drain (dec);
       GST_AUDIO_DECODER_STREAM_UNLOCK (dec);
 
+      if (dec->priv->ctx.had_input_data && !dec->priv->ctx.had_output_data) {
+        GST_ELEMENT_ERROR (dec, STREAM, DECODE,
+            ("No valid frames decoded before end of stream"),
+            ("no valid frames found"));
+      }
+
       /* Forward EOS because no buffer or serialized event will come after
        * EOS and nothing could trigger another _finish_frame() call. */
       ret = gst_audio_decoder_push_event (dec, event);
@@ -1849,9 +2135,6 @@ gst_audio_decoder_sink_eventfunc (GstAudioDecoder * dec, GstEvent * event)
 
     case GST_EVENT_CAPS:
     {
-      GstCaps *caps;
-
-      gst_event_parse_caps (event, &caps);
       ret = TRUE;
       dec->priv->do_caps = TRUE;
       gst_event_unref (event);
@@ -2529,9 +2812,10 @@ _gst_audio_decoder_error (GstAudioDecoder * dec, gint weight,
     GST_WARNING_OBJECT (dec, "error: %s", dbg);
   dec->priv->error_count += weight;
   dec->priv->discont = TRUE;
-  if (dec->priv->ctx.max_errors < dec->priv->error_count) {
-    gst_element_message_full (GST_ELEMENT (dec), GST_MESSAGE_ERROR,
-        domain, code, txt, dbg, file, function, line);
+  if (dec->priv->ctx.max_errors >= 0
+      && dec->priv->ctx.max_errors < dec->priv->error_count) {
+    gst_element_message_full (GST_ELEMENT (dec), GST_MESSAGE_ERROR, domain,
+        code, txt, dbg, file, function, line);
     return GST_FLOW_ERROR;
   } else {
     g_free (txt);
@@ -2632,8 +2916,9 @@ gst_audio_decoder_get_delay (GstAudioDecoder * dec)
  * @num: max tolerated errors
  *
  * Sets numbers of tolerated decoder errors, where a tolerated one is then only
- * warned about, but more than tolerated will lead to fatal error.  Default
- * is set to GST_AUDIO_DECODER_MAX_ERRORS.
+ * warned about, but more than tolerated will lead to fatal error. You can set
+ * -1 for never returning fatal errors. Default is set to
+ * GST_AUDIO_DECODER_MAX_ERRORS.
  */
 void
 gst_audio_decoder_set_max_errors (GstAudioDecoder * dec, gint num)
@@ -2937,7 +3222,7 @@ gst_audio_decoder_get_needs_format (GstAudioDecoder * dec)
 {
   gboolean result;
 
-  g_return_val_if_fail (GST_IS_AUDIO_DECODER (dec), 0);
+  g_return_val_if_fail (GST_IS_AUDIO_DECODER (dec), FALSE);
 
   GST_OBJECT_LOCK (dec);
   result = dec->priv->needs_format;
@@ -2996,6 +3281,7 @@ GstBuffer *
 gst_audio_decoder_allocate_output_buffer (GstAudioDecoder * dec, gsize size)
 {
   GstBuffer *buffer = NULL;
+  gboolean needs_reconfigure = FALSE;
 
   g_return_val_if_fail (size > 0, NULL);
 
@@ -3003,10 +3289,11 @@ gst_audio_decoder_allocate_output_buffer (GstAudioDecoder * dec, gsize size)
 
   GST_AUDIO_DECODER_STREAM_LOCK (dec);
 
+  needs_reconfigure = gst_pad_check_reconfigure (dec->srcpad);
   if (G_UNLIKELY (dec->priv->ctx.output_format_changed ||
           (GST_AUDIO_INFO_IS_VALID (&dec->priv->ctx.info)
-              && gst_pad_check_reconfigure (dec->srcpad)))) {
-    if (!gst_audio_decoder_negotiate (dec)) {
+              && needs_reconfigure))) {
+    if (!gst_audio_decoder_negotiate_unlocked (dec)) {
       GST_INFO_OBJECT (dec, "Failed to negotiate, fallback allocation");
       gst_pad_mark_reconfigure (dec->srcpad);
       goto fallback;
