@@ -63,8 +63,6 @@
  * ]| Capture video from a V4L device, and adjust the stream to 12.5 fps before
  * encoding to Ogg/Theora.
  * </refsect2>
- *
- * Last reviewed on 2006-09-02 (0.10.11)
  */
 
 #ifdef HAVE_CONFIG_H
@@ -200,8 +198,6 @@ gst_video_rate_class_init (GstVideoRateClass * klass)
    * GstVideoRate:skip-to-first:
    * 
    * Don't produce buffers before the first one we receive.
-   *
-   * Since: 0.10.25
    */
   g_object_class_install_property (object_class, PROP_SKIP_TO_FIRST,
       g_param_spec_boolean ("skip-to-first", "Skip to first buffer",
@@ -212,8 +208,6 @@ gst_video_rate_class_init (GstVideoRateClass * klass)
    * GstVideoRate:drop-only:
    *
    * Only drop frames, no duplicates are produced.
-   *
-   * Since: 0.10.36
    */
   g_object_class_install_property (object_class, PROP_DROP_ONLY,
       g_param_spec_boolean ("drop-only", "Only Drop",
@@ -226,8 +220,6 @@ gst_video_rate_class_init (GstVideoRateClass * klass)
    * Arrange for maximum framerate by dropping frames beyond a certain framerate,
    * where the framerate is calculated using a moving average over the
    * configured.
-   *
-   * Since: 0.10.36
    */
   g_object_class_install_property (object_class, PROP_AVERAGE_PERIOD,
       g_param_spec_uint64 ("average-period", "Period over which to average",
@@ -239,8 +231,6 @@ gst_video_rate_class_init (GstVideoRateClass * klass)
    * GstVideoRate:max-rate:
    *
    * maximum framerate to pass through
-   *
-   * Since: 0.10.36
    */
   g_object_class_install_property (object_class, PROP_MAX_RATE,
       g_param_spec_int ("max-rate", "maximum framerate",
@@ -482,8 +472,6 @@ gst_video_rate_setcaps (GstBaseTransform * trans, GstCaps * in_caps,
   gboolean ret = TRUE;
   gint rate_numerator, rate_denominator;
 
-  videorate = GST_VIDEO_RATE (trans);
-
   GST_DEBUG_OBJECT (trans, "setcaps called in: %" GST_PTR_FORMAT
       " out: %" GST_PTR_FORMAT, in_caps, out_caps);
 
@@ -779,6 +767,10 @@ gst_video_rate_sink_event (GstBaseTransform * trans, GstEvent * event)
       GST_DEBUG_OBJECT (videorate, "Got FLUSH_STOP");
       gst_video_rate_reset (videorate);
       break;
+    case GST_EVENT_GAP:
+      /* no gaps after videorate, ignore the event */
+      gst_event_unref (event);
+      return TRUE;
     default:
       break;
   }
