@@ -18,9 +18,7 @@
  * Boston, MA 02110-1301, USA.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <unistd.h>
 
 #include <gst/check/gstcheck.h>
 #include <gst/video/video-overlay-composition.h>
@@ -55,8 +53,7 @@ static GstPad *myvideosrcpad, *mytextsrcpad, *mysinkpad;
     "format = (string) I420"
 
 #define VIDEO_CAPS_TEMPLATE_WITH_FEATURE_STRING                              \
-    "video/x-raw(" GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY ", "                \
-    GST_CAPS_FEATURE_META_GST_VIDEO_OVERLAY_COMPOSITION "), "                \
+    "video/x-raw(" GST_CAPS_FEATURE_META_GST_VIDEO_OVERLAY_COMPOSITION "), " \
     "format = (string) I420;"                                                \
     "video/x-raw, "                                                          \
     "format = (string) I420;"
@@ -111,11 +108,6 @@ GST_STATIC_PAD_TEMPLATE ("sink",
     GST_STATIC_CAPS (UNSUPPORTED_VIDEO_CAPS_TEMPLATE_WITH_FEATURE_STRING)
     );
 
-static GstStaticPadTemplate sinktemplate_any = GST_STATIC_PAD_TEMPLATE ("sink",
-    GST_PAD_SINK,
-    GST_PAD_ALWAYS,
-    GST_STATIC_CAPS_ANY);
-
 static GstStaticPadTemplate unsupported_video_srctemplate =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
@@ -181,7 +173,7 @@ notgst_check_setup_src_pad2 (GstElement * element,
   ASSERT_OBJECT_REFCOUNT (srcpad, "srcpad", 1);
 
   if (!(sinkpad = gst_element_get_static_pad (element, sink_template_name)))
-    sinkpad = gst_element_request_pad_simple (element, sink_template_name);
+    sinkpad = gst_element_get_request_pad (element, sink_template_name);
   fail_if (sinkpad == NULL, "Could not get sink pad from %s",
       GST_ELEMENT_NAME (element));
   ASSERT_OBJECT_REFCOUNT (sinkpad, "sinkpad", 2);
@@ -206,7 +198,7 @@ notgst_check_teardown_src_pad2 (GstElement * element,
 
   /* clean up floating src pad */
   if (!(sinkpad = gst_element_get_static_pad (element, sink_template_name)))
-    sinkpad = gst_element_request_pad_simple (element, sink_template_name);
+    sinkpad = gst_element_get_request_pad (element, sink_template_name);
   ASSERT_OBJECT_REFCOUNT (sinkpad, "sinkpad", 2);
   srcpad = gst_pad_get_peer (sinkpad);
 
@@ -363,34 +355,6 @@ create_text_buffer (const gchar * txt, GstClockTime ts, GstClockTime duration)
   return buffer;
 }
 
-static gboolean
-_test_textoverlay_check_caps_has_feature (GstElement * textoverlay,
-    const gchar * padname, const gchar * feature)
-{
-  GstPad *pad;
-  GstCaps *caps;
-  GstCapsFeatures *f;
-  gboolean ret;
-
-  pad = gst_element_get_static_pad (textoverlay, padname);
-  fail_unless (pad != NULL);
-
-  caps = gst_pad_get_current_caps (pad);
-  fail_unless (caps != NULL);
-
-  gst_object_unref (pad);
-
-  f = gst_caps_get_features (caps, 0);
-  if (f != NULL) {
-    ret = gst_caps_features_contains (f, feature);
-  } else {
-    ret = FALSE;
-  }
-
-  gst_caps_unref (caps);
-  return ret;
-}
-
 static void
 cleanup_textoverlay (GstElement * textoverlay)
 {
@@ -684,78 +648,6 @@ GST_START_TEST (test_video_passthrough_with_feature_and_unsupported_caps)
 
 GST_END_TEST;
 
-
-GST_START_TEST (test_video_render_with_any_features_and_no_allocation_meta)
-{
-  GstElement *textoverlay;
-  GstBuffer *inbuffer, *outbuffer;
-  GstCaps *incaps, *outcaps;
-  GstVideoOverlayCompositionMeta *comp_meta;
-
-  textoverlay =
-      setup_textoverlay_with_templates (&video_srctemplate,
-      NULL, &sinktemplate_any, FALSE);
-
-  /* set static text to render */
-  g_object_set (textoverlay, "text", "XLX", NULL);
-
-  fail_unless (gst_element_set_state (textoverlay,
-          GST_STATE_PLAYING) == GST_STATE_CHANGE_SUCCESS,
-      "could not set to playing");
-
-  incaps = create_video_caps (VIDEO_CAPS_STRING);
-  gst_check_setup_events_textoverlay (myvideosrcpad, textoverlay, incaps,
-      GST_FORMAT_TIME, "video");
-  inbuffer = create_black_buffer (incaps);
-  gst_caps_unref (incaps);
-  ASSERT_BUFFER_REFCOUNT (inbuffer, "inbuffer", 1);
-
-  GST_BUFFER_TIMESTAMP (inbuffer) = 0;
-  GST_BUFFER_DURATION (inbuffer) = GST_SECOND / 10;
-
-  /* take additional ref to keep it alive */
-  gst_buffer_ref (inbuffer);
-  ASSERT_BUFFER_REFCOUNT (inbuffer, "inbuffer", 2);
-
-  /* pushing gives away one of the two references we have ... */
-  fail_unless (gst_pad_push (myvideosrcpad, inbuffer) == GST_FLOW_OK);
-
-  /* should have been dropped in favour of a new writable buffer */
-  ASSERT_BUFFER_REFCOUNT (inbuffer, "inbuffer", 1);
-  fail_unless_equals_int (g_list_length (buffers), 1);
-  outbuffer = GST_BUFFER_CAST (buffers->data);
-  outcaps = gst_pad_get_current_caps (mysinkpad);
-  fail_unless (outbuffer != inbuffer);
-
-  /* output buffer should have rendered text */
-  fail_if (buffer_is_all_black (outbuffer, outcaps));
-  gst_caps_unref (outcaps);
-
-  /* output buffer should not have the composition meta */
-  comp_meta = gst_buffer_get_video_overlay_composition_meta (outbuffer);
-  fail_unless (comp_meta == NULL);
-
-  fail_unless (GST_BUFFER_TIMESTAMP (outbuffer) == 0);
-  fail_unless (GST_BUFFER_DURATION (outbuffer) == (GST_SECOND / 10));
-
-  /* output caps shouldn't have the composition meta */
-  fail_if (_test_textoverlay_check_caps_has_feature (textoverlay, "src",
-          GST_CAPS_FEATURE_META_GST_VIDEO_OVERLAY_COMPOSITION));
-
-  /* and clean up */
-  g_list_foreach (buffers, (GFunc) gst_mini_object_unref, NULL);
-  g_list_free (buffers);
-  buffers = NULL;
-  ASSERT_BUFFER_REFCOUNT (inbuffer, "inbuffer", 1);
-
-  /* cleanup */
-  cleanup_textoverlay (textoverlay);
-  gst_buffer_unref (inbuffer);
-}
-
-GST_END_TEST;
-
-
 GST_START_TEST (test_video_render_static_text)
 {
   GstElement *textoverlay;
@@ -896,7 +788,7 @@ GST_START_TEST (test_video_waits_for_text)
   fail_unless_equals_int (g_list_length (buffers), 1);
 
   /* text should still be stuck in textoverlay */
-  ASSERT_MINI_OBJECT_REFCOUNT (gst_buffer_peek_memory (tbuf, 0), "tbuf-mem", 2);
+  ASSERT_BUFFER_REFCOUNT (tbuf, "tbuf", 2);
 
   /* there should be no text rendered */
   outbuffer = GST_BUFFER_CAST (buffers->data);
@@ -919,7 +811,7 @@ GST_START_TEST (test_video_waits_for_text)
   fail_unless_equals_int (g_list_length (buffers), 2);
 
   /* text should still be stuck in textoverlay */
-  ASSERT_MINI_OBJECT_REFCOUNT (gst_buffer_peek_memory (tbuf, 0), "tbuf-mem", 2);
+  ASSERT_BUFFER_REFCOUNT (tbuf, "tbuf", 2);
 
   /* there should be text rendered */
   outbuffer = GST_BUFFER_CAST (buffers->next->data);
@@ -947,7 +839,7 @@ GST_START_TEST (test_video_waits_for_text)
   fail_unless (gst_pad_push (myvideosrcpad, inbuffer) == GST_FLOW_OK);
 
   /* but the text should no longer be stuck in textoverlay */
-  ASSERT_MINI_OBJECT_REFCOUNT (gst_buffer_peek_memory (tbuf, 0), "tbuf-mem", 1);
+  ASSERT_BUFFER_REFCOUNT (tbuf, "tbuf", 1);
 
   /* video buffer should have gone through after newsegment event */
   fail_unless_equals_int (g_list_length (buffers), 3);
@@ -971,6 +863,7 @@ GST_START_TEST (test_video_waits_for_text)
   thread = g_thread_try_new ("gst-check",
       test_video_waits_for_text_shutdown_element, textoverlay, NULL);
   fail_unless (thread != NULL);
+  g_thread_unref (thread);
 
   GST_LOG ("pushing video buffer 4");
   fail_unless (gst_pad_push (myvideosrcpad, inbuffer) == GST_FLOW_FLUSHING);
@@ -981,7 +874,6 @@ GST_START_TEST (test_video_waits_for_text)
   buffers = NULL;
 
   /* cleanup */
-  g_thread_join (thread);
   cleanup_textoverlay (textoverlay);
 
   /* give up our ref, textoverlay should've cleared its queued buffer by now */
@@ -1150,8 +1042,6 @@ textoverlay_suite (void)
   tcase_add_test (tc_chain, test_video_passthrough_with_feature);
   tcase_add_test (tc_chain,
       test_video_passthrough_with_feature_and_unsupported_caps);
-  tcase_add_test (tc_chain,
-      test_video_render_with_any_features_and_no_allocation_meta);
   tcase_add_test (tc_chain, test_video_render_static_text);
   tcase_add_test (tc_chain, test_render_continuity);
   tcase_add_test (tc_chain, test_video_waits_for_text);
